/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
 *                                                                         *
 ***************************************************************************/

#include "KCDemuSystemTray.h"
#include "KCDemuSystemTray.moc"

//Load icons and make default settings
KCDemuSystemTray::KCDemuSystemTray(QWidget* parent,  const char* name) {
	this->mountedIcon = KGlobal::iconLoader()->loadIcon( "cdrom_mount", KIcon::Small );
	this->unmountedIcon = KGlobal::iconLoader()->loadIcon( "cdrom_unmount", KIcon::Small );

	this->pFirstDeviceMenu = NULL;

	//KDE 3.2 stuff
//	this->mountedIcon = this->loadIcon( "cdrom_mount" );
//	this->unmountedIcon = this->loadIcon( "cdrom_unmount" );

	this->setIcon( this->unmountedIcon );
	this->createMenus();
}

//Delete all created objects
KCDemuSystemTray::~KCDemuSystemTray() {
	//Clean up
//	delete this->deviceMenu;
	delete this->settingsMenu;
	delete this->trayLeftMenu;
	delete this->trayRightMenu;
	delete this->pFirstDeviceMenu;
}

//Called by the device menus once mount / umount was successfull
void KCDemuSystemTray::changeState( int numDev, bool bMounted ) {
	for( KCDemuDeviceMenuStruct *tmp = this->pFirstDeviceMenu; tmp != NULL; tmp = tmp->pNext ) {
		if( tmp->numDev == numDev ) {
			if( bMounted ) {
				this->trayLeftMenu->changeItem( tmp->deviceMenuID, this->mountedIcon, this->trayLeftMenu->text( tmp->deviceMenuID ) );
			}
			else {
				this->trayLeftMenu->changeItem( tmp->deviceMenuID, this->unmountedIcon, this->trayLeftMenu->text( tmp->deviceMenuID ) );
			}
			break;
		}
	}
}

//Unmount all images
void KCDemuSystemTray::unmountAllImages() {
	for( KCDemuDeviceMenuStruct *tmp = this->pFirstDeviceMenu; tmp != NULL; tmp = tmp->pNext ) {
		tmp->deviceMenu->unmountCDImage();
	}
}

//Show the right menu when clicked
void KCDemuSystemTray::mousePressEvent (QMouseEvent *e) {
	switch( e->button() ) {
		case LeftButton:
			this->trayLeftMenu->popup( e->globalPos() );
			break;
		case RightButton:
			this->trayRightMenu->popup( e->globalPos() );
			break;
		default:
			break;
	}
}

//Create the default menus
int KCDemuSystemTray::createMenus () {
	int insertID = -1;

	this->trayLeftMenu = new QPopupMenu (0, "KCDemuLeftPopupMenu" );
	this->trayRightMenu = new QPopupMenu (0, "KCDemuRightPopupMenu" );

//	this->deviceMenu = new KCDemuDeviceMenu( this->trayLeftMenu, "KCDemuDeviceMenu" );
//	this->deviceMenuID = this->trayLeftMenu->insertItem( this->unmountedIcon, "Device 0", this->deviceMenu );
	createDeviceMenu( 0 );
	createDeviceMenu( 1 );
	createDeviceMenu( 2 );
	createDeviceMenu( 3 );
	createDeviceMenu( 4 );
	createDeviceMenu( 5 );
	createDeviceMenu( 6 );
	createDeviceMenu( 7 );
//	this->trayLeftMenu->setItemEnabled( deviceMenuID, true);
//	QObject::connect( this->deviceMenu, SIGNAL( CDImageOpened( int, bool ) ), this, SLOT( changeState( int, bool ) ) );

//	printf("insertID: %d\n", insertID);

//	this->settingsMenu = new KCDemuSettingsMenu( this->trayRightMenu, "KCDemuSettingsMenu" );
//	insertID = this->trayRightMenu->insertItem( "Settings", this->settingsMenu );
//	this->trayRightMenu->setItemEnabled( insertID, false);

	this->trayRightMenu->insertItem( KGlobal::iconLoader()->loadIcon( "exit", KIcon::Small), "Quit", KApplication::kApplication(), SLOT( quit() ) );
	this->trayLeftMenu->insertItem( "Unmount all", this, SLOT( unmountAllImages() ) );

//	printf("insertID: %d\n", insertID);
}

int KCDemuSystemTray::createDeviceMenu( int numDev ) {
	KCDemuDeviceMenuStruct *newMenuHolder = new KCDemuDeviceMenuStruct;

	QString MenuName( "Device " );
	MenuName.append( QString::number( numDev ) );

	newMenuHolder->deviceMenu = new KCDemuDeviceMenu( this->trayLeftMenu, "KCDemuDeviceMenu",  numDev );
	newMenuHolder->deviceMenuID = this->trayLeftMenu->insertItem( this->unmountedIcon, MenuName, newMenuHolder->deviceMenu );
	newMenuHolder->numDev = numDev;

	//Connect to slot
	QObject::connect( newMenuHolder->deviceMenu, SIGNAL( CDImageOpened( int, bool ) ), this, SLOT( changeState( int, bool ) ) );

	newMenuHolder->pNext = this->pFirstDeviceMenu;
	this->pFirstDeviceMenu = newMenuHolder;

	//Finally update status
	newMenuHolder->deviceMenu->checkStatus();

	return newMenuHolder->deviceMenuID;
}

int KCDemuSystemTray::setIcon (QPixmap newIcon) {
	setPixmap(newIcon);
}
